Package: catIrt
Type: Package
Version: 0.5.1
Title: Simulate IRT-Based Computerized Adaptive Tests
Authors@R: c(
  person(given = "Steven",
         family = "Nydick",
         role = c("cre", "aut"),
         email = "swnydick@gmail.com",
         comment = c(ORCID = "0000-0002-2908-1188")))
Maintainer: Steven Nydick <swnydick@gmail.com>
URL: https://github.com/swnydick/catIrt
BugReports: https://github.com/swnydick/catIrt/issues
Depends: R (>= 2.11.0), numDeriv (>= 2012.3-1)
Suggests: irtoys, ltm, catR
Description: Functions designed to simulate data that conform to basic
          unidimensional IRT models (for now 3-parameter binary response models
          and graded response models) along with Post-Hoc CAT simulations of
          those models given various item selection methods, ability estimation
          methods, and termination criteria. See
          Wainer (2000) <doi:10.4324/9781410605931>,
          van der Linden & Pashley (2010) <doi:10.1007/978-0-387-85461-8_1>,
          and Eggen (1999) <doi:10.1177/01466219922031365> for more details.
License: GPL (>= 2)
LazyLoad: yes
NeedsCompilation: yes
Packaged: 2022-05-25 22:16:59 UTC; stevennydick
Author: Steven Nydick [cre, aut] (<https://orcid.org/0000-0002-2908-1188>)
Repository: CRAN
Date/Publication: 2022-05-25 22:50:10 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 17:22:54 UTC; unix
Archs: catIrt.so.dSYM
